import java.util.Scanner;
/**
 * 183.592 Programmierpraxis TU Wien WS2014/15 H.Moritsch
 * Random-Surfer Modell zur Berechnung des PageRank einer Webseite
 * s. R. Sedgewick, K. Wayne. Introduction to Programming in 
 * Java: An Interdisciplinary Approach. Addison-Wesley, 2008.
 * Aufruf: java RandomSurfer < randomsurfer.dat
 */
public class RandomSurfer {
    public static void main(String[] args) {

        Scanner scanner = new Scanner(System.in);

        /* 1. Einlesen der Webstruktur (Seiten und Links) */

        // Anzahl der Seiten
        Page[] pages = new Page[scanner.nextInt()];

        // für jede Seite (i):
        for (int i=0; i<pages.length; i++) {

            // Erzeugen des Page-Objekts mit URL und Anzahl der Links auf der Seite
            pages[i] = new Page(scanner.next(), scanner.nextInt());

        }

        // für jede Seite (i):
        for (int i=0; i<pages.length; i++) {
            System.out.print((i+1) + ". " + pages[i].getURL() + " -> ");

            // für jeden Link auf dieser Seite (j):
            for (int j=0; j<pages[i].getDegree(); j++) {

                // Nummer der Seite, auf die der Link verweist (k)
                int k = scanner.nextInt();      

                // Eintragen des Links: auf der i.ten Seite verweist 
                // der j.te Link auf das Page-Objekt "pages[k-1]"
                pages[i].setLink( j, pages[k-1] ); 

                System.out.print(pages[k-1].getURL() + " ");

                }
            System.out.println();
            }

        /* 2. Random-Surfer surft */

        Page currentPage = pages[0];                // startet mit der ersten Seite

        System.out.println("start page: "+currentPage.getURL());

        int totalVisits = scanner.nextInt();        // Anzahl der durchzuführenden Seitenaufrufe

        int visitCount = 1;                         // zählt Anzahl der Seitenaufrufe insgesamt     
        while ( visitCount++ < totalVisits ) {

            /* Random-Surfer sitzt vor dem Browser und hat eine Seite vor sich. 
               Welche Seite sieht er sich als nächste an? 
            */

            if (Math.random() < 0.90) {
                // in 90% aller der Fälle wählt er irgendeinen der *Links* auf 
                // der aktuellen Seite (nach Zufallsprinzip)

                // k ist eine Zufallszahl zwischen 0 und (Anzahl der Links - 1)
                int k = (int)( Math.random() * currentPage.getDegree());

                // gehe zur Seite, auf die dieser Link verweist
                currentPage = currentPage.getLink(k);

                // System.out.println("90%: "+currentPage.getURL());

            }

            else {
                // in den restlichen 10% der Fälle wählt er irgendeine der 
                // vorhandenen *Seiten* (nach Zufallsprinzip)

                // k ist eine Zufallszahl zwischen 0 und (Anzahl der Seiten - 1)
                int k = (int)( Math.random() * pages.length );

                // gehe zu dieser Seite
                currentPage = pages[k];

                // System.out.println("10%: "+currentPage.getURL());

            }

            currentPage.incrementCount();           // Hitcount der neuen Seite wird erhöht

        }

        /* 3. Ausgabe der Hitcounts */

        System.out.println("total visits: " + totalVisits);
          
        // für jede Seite (i):
        for (int i=0; i<pages.length; i++) 
            System.out.println( pages[i].getURL() + ": " + 
                              ( pages[i].getCount() / (totalVisits/100.0) ) + " % of visits" );

    }

}
