import java.util.Scanner;
/**
 * 183.592 Programmierpraxis TU Wien WS2014/15 H.Moritsch
 * Random-Surfer Modell zur Berechnung des PageRank einer Webseite
 * mit Anzeige der Datenstruktur und des Programmablaufs
 * s. R. Sedgewick, K. Wayne. Introduction to Programming in 
 * Java: An Interdisciplinary Approach. Addison-Wesley, 2008.
 * Aufruf: java RandomSurfer < randomsurfer.dat
 */
public class RandomSurferV2 {
    public static void main(String[] args) {

        Scanner scanner = new Scanner(System.in);

        /* 1. Einlesen der Webstruktur (Seiten und Links) */

        // Anzahl der Seiten
        Page[] pages = new Page[scanner.nextInt()];

        System.out.println();
        System.out.println("=============== web structure =======================");

        // für jede Seite (i):
        for (int i=0; i<pages.length; i++) {

            // Erzeugen des Page-Objekts mit URL und Anzahl der Links auf der Seite
            pages[i] = new Page(scanner.next(), scanner.nextInt());

        }

        // für jede Seite (i):
        for (int i=0; i<pages.length; i++) {
            System.out.print((i+1) + ". " + pages[i].getURL() + " -> ");

            // für jeden Link auf dieser Seite (j):
            for (int j=0; j<pages[i].getDegree(); j++) {

                // Nummer der Seite, auf die der Link verweist (k)
                int k = scanner.nextInt();      

                // Eintragen des Links: auf der i.ten Seite verweist 
                // der j.te Link auf das Page-Objekt "pages[k-1]"
                pages[i].setLink( j, pages[k-1] ); 

                System.out.print(pages[k-1].getURL() + " ");

            }
            System.out.println();
        }
        System.out.println("=====================================================");

        /* 1.a Anzeige der Datenstruktur */

        System.out.println();
        System.out.println("--------------- data structure ----------------------");

        System.out.println("pages.length = " + pages.length);
        // für jede Seite (i):
        for (int i=0; i<pages.length; i++) {

            System.out.println();
            System.out.println("pages[" + i + "] = <" + pages[i] + ">");
            System.out.println("pages[" + i + "].url = \"" + pages[i].getURL() + "\"");

            System.out.println("pages[" + i + "].links.length = " + pages[i].getDegree());
            for (int j=0; j<pages[i].getDegree(); j++) {
                Page p = pages[i].getLink(j);
                System.out.println("pages[" + i + "].links["+ j + "] = <" + p + "> (\"" + p.getURL() + "\")");
            }

            System.out.println("pages[" + i + "].count = " + pages[i].getCount());
        }

        System.out.println();
        System.out.println("-----------------------------------------------------");
        System.out.println();

        /* 2. Random-Surfer surft */

        int totalVisits = scanner.nextInt();        // Anzahl der durchzuführenden Seitenaufrufe

        Page currentPage = pages[0];                // startet mit der ersten Seite

        System.out.println("start page: "+currentPage.getURL());

        int visitCount = 0;                         // zählt Anzahl der Seitenaufrufe insgesamt     
        while ( visitCount++ < totalVisits ) {

            // scanner.nextLine();                  // nur im interaktiven Modus

            System.out.println();
            System.out.println("visit nr." + visitCount + ": " + currentPage.getURL());

            /* Random-Surfer sitzt vor dem Browser und hat eine Seite vor sich. 
               Welche Seite sieht er sich als nächste an? 
            */

            if (Math.random() < 0.90) {
                // in 90% aller der Fälle wählt er irgendeinen der *Links* auf 
                // der aktuellen Seite (nach Zufallsprinzip)

                // k ist eine Zufallszahl zwischen 0 und (Anzahl der Links - 1)
                int k = (int)( Math.random() * currentPage.getDegree());

                System.out.print("click link " + (k+1) + " (of " + currentPage.getDegree() + ") = ");

                // gehe zur Seite, auf die dieser Link verweist
                currentPage = currentPage.getLink(k);

            }

            else {
                // in den restlichen 10% der Fälle wählt er irgendeine der 
                // vorhandenen *Seiten* (nach Zufallsprinzip)

                // k ist eine Zufallszahl zwischen 0 und (Anzahl der Seiten)
                int k = (int)( Math.random() * pages.length );

                System.out.print("type page address:    ");

                // gehe zu dieser Seite
                currentPage = pages[k];

            }

            System.out.println(currentPage.getURL());

            currentPage.incrementCount();           // Hitcount der neuen Seite wird erhöht

        }

        /* 3. Ausgabe der Hitcounts */

        System.out.println();
        System.out.println("total visits: " + totalVisits);
          
        // für jede Seite (i):
        for (int i=0; i<pages.length; i++) 
            System.out.println( "page " + pages[i].getURL() + ": " + pages[i].getCount()
                + " visits (" + ( pages[i].getCount() / (totalVisits/100.0) ) + " %)");

    }

}
