import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
/**
 * 183.592 Programmierpraxis TU Wien WS2014/15 H.Moritsch
 * Ein Text-Objekt repräsentiert einen mehrzeiligen Text.
 * Dieser wird als eine Liste von Zeilen-Objekten dargestellt.
 * Es kommen nur Texte vor, die aus mindestens einer Zeile bestehen 
 * und keine Leerzeilen enthalten.
 */
class Text {

    private ArrayList<Line> lines;                  // die Zeilen des Texts

    private HashMap<String, Integer> wordCounts;    // Tabelle der Wortvorkommnisse

    /**
    * Konstruktor erzeugt leeren Text
    */
    Text() {
        lines = new ArrayList<Line>();
    }

    /**
    * Fügt eine Zeile am Ende des Texts an.
    * @param line anzufügenden Zeile
    */
    public void addLine(Line line) {
        lines.add(line);
    }

    /**
    * @return Liefert Anzahl der Zeilen
    */
    public int numberOfLines() {
        return lines.size();
    }

    /**
    * @return Liefert Iterator über die Zeilen des Texts
    */
    public Iterator<Line> getLineIterator() {
        return lines.iterator();
        }

    /**
    * @return Liefert die Tabelle der Wortvorkommnisse
    */
    public HashMap<String, Integer> getWordCounts() {
        return wordCounts;
    }

    /**
    * @return Liefert Iterator über alle Wörter des Texts
    */
    public Iterator<String> getWordIterator() {

        // Aufbau einer Liste aller Wörter
        ArrayList<String> allWords = new ArrayList<String>();

        for ( Line line : lines ) {         // alle Zeilen, mittels for-each 
            Iterator<String> wIt = line.getIterator();  
            while (wIt.hasNext()) {         // alle Wörter der Zeile, mittels Iterator
                                            // (for-each nicht möglich, da words in Line privat)
                String word = wIt.next();   
                allWords.add(word);         // hinzufügen
            }
        }

        return allWords.iterator();         // Iterator der Wörterliste
    }

    /**
    * @return Liefert Anzahl der Wörter
    */
    public int numberOfWords() {
        int n = 0;
        for ( Line line : lines )           // über alle Zeilen
            n += line.numberOfWords();      // Anzahl Wörter pro Zeile
        return n;
        }

    /**
    * @return Liefert Anzahl der Wörter, mittels getWordIterator
    */
    public int numberOfWordsIt() {
        int n = 0;
        Iterator<String> wIt = getWordIterator();
        while (wIt.hasNext()) {          // über alle Wörter
            wIt.next();                  // nächstes Wort
            n++;
            }
        return n;
        }

    /**
    * @return Liefert Anzahl der voneinander verschiedenen Wörter
    */
    public int numberOfUniqueWords() {
        HashSet <String> wordSet = new HashSet<String>();
        Iterator<String> wIt     = getWordIterator();

        while (wIt.hasNext()) {         // über alle Wörter
            String word = wIt.next();   
            wordSet.add(word);          // Wort zur Menge hinzufügen
        }

        return wordSet.size();          // Anzahl der Elemente der Menge
    }

    /**
    * Berechnet für jedes im Text auftretende Wort die Anzahl seiner Vorkommnisse
    * und trägt sie in wordCounts ein
    */
    public void calculateWordCounts() {
        wordCounts = new HashMap<String, Integer>();
        Iterator<String> wIt = getWordIterator();

        while (wIt.hasNext()){      // über alle Worter des Texts
            String w = wIt.next();
            wordCounts.put(w, wordCounts.get(w)==null ? 1 :     // Wort kommt zum 1.Mal vor
                    wordCounts.get(w)+1 );                      // Wort kommt ein weiteres Mal vor
            }
        }

    /**
    * Ersetzt ein Wort im ganzen Text durch ein anderes
    * @param word zu ersetzendes ("altes") Wort
    * @param newWord ersetzendes ("neues") Wort
    */
    public void substitute(String word, String newWord) {

        for ( Line line : lines )           // über alle Zeilen
            line.substitute(word, newWord); // ersetze in Zeile
    }

    /**
    * Erstellt eine Kopie eines Texts, in der jede Zeile nach 
    * einer bestimmten Zeichenposition endet
    * @param pos ab dieser Zeichenposition beginnt kein Wort mehr. 
    * Das letzte Wort der Zeile kann aber über diese Position hinausragen.
    */
    public Text format(int pos) {
        Iterator<String> wIt = getWordIterator();
        Line line;
        int c = 0;
        Text text = new Text();

        line = new Line();
        while (wIt.hasNext()){
            String word = wIt.next();
            line.addWord(word);
            c += word.length()+1;
            if (c >= pos) {
                text.addLine(line);
                line = new Line();
                c = 0;
            }
        }
        if (line.numberOfWords()>0)  
            text.addLine(line);
    
        return text;
    }

}
