package sensor;
 
import java.util.Date;
/**
 * 183.592 Programmierpraxis TU Wien WS2014/15 H.Moritsch
 * Punkt mit Sensoren
 * Diese Klasse definiert einen Sensor. Das Auslesen des Wertes des Sensors -
 * readValue() - wird nicht implementiert, daher ist diese Klasse abstrakt.
 * Diese Klasse enthält die Variablen eines Sensors sowie den Konstruktor.
 **/
public abstract class AbstractSensor implements Sensor {

    private final String name;  // der Name des Sensors
    private Date date;          // der Zeitpunkt des letztmaligen Auslesens mittels read()
    private double value;       // der zuletzt mittels readValue() ausgelesene Wert: 
                                // wird mittels readValue() in Unterklassen gesetzt

    // Konstruktor - setzt den Namen
    // wird in den Unterklassen aufgerufen
    protected AbstractSensor(String name) {
        this.name = name;
    }

    // abstrakte Methode zum Auslesen des Wertes
    // hängt von der Art des Sensors ab
    // wird erst in Unterklassen implementiert
    protected abstract void readValue();

    // liefert den Namen    
    public String getName() {
        return name;
    }

    // Auslesen des Sensors
    public void read() {
        date = new Date();      // aktuelle Zeit speichern (unabhängig von Art des Sensors)
        readValue();            // Aufruf der abstrakten Methode
    }

    // liefert den Zeitpunkt des letzten Auslesens
    public Date getDate() {
        return date;
    }

    // liefert den zuletzt ausgelesenen Wert
    public double getValue() {
        return value;
    }

    // setzt den zuletzt ausgelesenen Wert
    // in einer Unterklasse
    protected void setValue(double value) {
        this.value = value;
    }

}
