import java.util.ArrayList;
/** 
 * 183.592 Programmierpraxis TU Wien WS2014/15 H.Moritsch
 * Ein PersonNode ist ein Knoten in einer verketteten Liste. Er repräsentiert 
 * einen einzelnen Personeneintrag. An einen Knoten kann ein weiterer Knoten 
 * angehängt werden, wodurch eine Liste von PersonNode-Objekten entsteht. 
 * Jeder Knoten ist der Startknoten der Liste, die aus "seinem" Personeneintrag
 * und der Restliste (die über next referenziert wird), besteht. Der Startknoten 
 * hat die Position (Index) 0.
 */
public class PersonNode {

    Person person;
    PersonNode next;

    /**
     * Konstruktor: initialisiert diesen Knoten mit einer Person und der
     * Referenz zum nächsten Knoten.
     */
    public PersonNode(Person person, PersonNode next) {
        
        this.person = person;
        this.next = next;
    }

    /**
     * Gibt die Liste aus
     */
    public void showIterative() {
        int i=0;
        for (PersonNode pn = this; pn != null; pn = pn.next) {
            System.out.println(i + " " + pn.person);
            i++;
        }
    }

    /**
     * Gibt die Liste aus (rekursiv, ohne Index)
     */
    public void show() {

        System.out.println(this.person);    // gib Person-Objekt dieses Knotens aus

        if (this.next != null)              // wenn dieser Knoten nicht der Endknoten ist,
            this.next.show();               // gib Restliste aus
    }

    /**
     * Gibt die Liste aus (rekursiv, mit Index)
     */
    public void showWithIndex(int i) {

        System.out.println(i + " " + this.person);

        if (this.next != null)              
            this.next.showWithIndex( i + 1 );   // Erhöhen des Index
    }

    /**
     * Anzahl der Studierenden 
     */
    public int numberOfStudents() {

        // s: 1 wenn Student, andernfalls 0
        int s = this.person instanceof Student ? 1 : 0; 
        
        if (this.next == null)              // wenn dieser Knoten der Endknoten ist,
            return s;                       // gib s zurück
                
        // andernfalls gib die um s erhöhte Anzahl der Studierenden in der Restliste zurück
        return s + this.next.numberOfStudents();
    }

 
    /**
     * Liefert den Personeneintrag an einer bestimmten Position
     */
    public Person get(int index) {
        
        if (index == 0)                     // wenn dieser Knoten (Position 0) gefragt ist 
            return this.person;             // gib diesen Knoten zurück
        
        return this.next.get( index-1 );    // andernfalls gib den Personeneintrag an der um 1 
                                            // verminderten Position in der Restliste zurück
    }
   
    /**
     * Liefert die Position der ersten Person mit einem bestimmten Namen in der Liste
     * Falls keine Person mit diesem Namen gefunden wird, wird -1 zurückgegeben.
     */
    public int indexOf(String name) {
        
        if (name.equals(this.person.getName())) // wenn der Name in diesem Knoten enthalten ist,
            return 0;                           // gib als Position 0 zurück
        

        // andernfalls (Name ist in diesem Knoten nicht enthalten):

        if (this.next == null)                  // wenn dieser Knoten der Endknoten ist,
            return -1;                          // gib -1 zurück ("Name nicht enthalten")

        // andernfalls ( Name ist in diesem Knoten nicht enthalten UND 
        // dieser Knoten ist nicht der Endknoten):

        int index = this.next.indexOf(name);    // ermittle die Position in der Restliste

        return index < 0 ? index : index + 1;   // in der Restliste nicht enthalten -> -1,
                                    // andernfalls: um 1 erhöhte Position in der Restliste 
    }
 
    /**
     * Liefert eine Liste (ArrayList) mit den Matrikelnummern aller Studierenden 
     */
    public ArrayList<Integer> getMatnrList() {

        ArrayList<Integer> aList;               // ArrayList, die zurückgegeben wird

        if (this.next == null)                  // wenn dieser Knoten der Endknoten ist,
            aList = new ArrayList<Integer>();   // erhalte eine neue leere ArrayList
        else                                    // andernfalls:
            aList = this.next.getMatnrList();   // erhalte ArrayList für die Restliste
    
        if (this.person instanceof Student) {   // wenn dieser Knoten einen Studierendeneintrag 
            int matnr = ((Student) person).getMatnr();  // enthält:
            aList.add(matnr);                           // füge Matrikelnummer an ArrayList an
        }
        return aList;
    }
           
}
